﻿Imports System.ComponentModel
Imports Wisej.Web

Public Class Page1

    Private mColumnsProperties As System.Collections.Generic.Dictionary(Of Integer, PropertyDescriptor)

    Private mImageItalia As System.Drawing.Image
    Private mImageBrasile As System.Drawing.Image
    Private mImageArgentina As System.Drawing.Image

    Private Sub Page1_Load(sender As Object, e As EventArgs) Handles Me.Load

        Inizializza()

        Button1.Image = mImageItalia

        Dim wAnagrafiche As New Anagrafiche
        wAnagrafiche.Carica()
        DGV.AutoGenerateColumns = False
        DGV.DataSource = wAnagrafiche
    End Sub

    Private Sub Inizializza()
        mImageItalia = GetImage("italia.png")
        mImageBrasile = GetImage("brasile.png")
        mImageArgentina = GetImage("argentina.png")
    End Sub

    Private Sub DGV_DataBindingComplete(sender As Object, e As DataGridViewBindingCompleteEventArgs) Handles DGV.DataBindingComplete
        mColumnsProperties = New System.Collections.Generic.Dictionary(Of Integer, PropertyDescriptor)

        For Each wCol As DataGridViewColumn In DGV.Columns
            Dim wPropDesc As System.ComponentModel.PropertyDescriptor
            If wCol.DataPropertyName Is Nothing Then
                mColumnsProperties.Add(wCol.Index, Nothing)
                Continue For
            End If
            wPropDesc = GetPropertyDescriptor(DGV, Convert.ToString(wCol.DataPropertyName))
            If wPropDesc Is Nothing Then
                mColumnsProperties.Add(wCol.Index, Nothing)
                Continue For
            End If
            mColumnsProperties.Add(wCol.Index, wPropDesc)
        Next
    End Sub

    Private Shared Function GetPropertyDescriptor(Lista As DataGridView, strPropertyName As String) As System.ComponentModel.PropertyDescriptor
        Dim wPropertyDescriptor As System.ComponentModel.PropertyDescriptor = Nothing
        Dim wLista = TryCast(Lista.DataSource, System.Collections.IList)
        If wLista Is Nothing Then Return Nothing
        Dim wProps = ListBindingHelper.GetListItemProperties(wLista)
        If wProps Is Nothing Then Return Nothing
        wPropertyDescriptor = wProps.Item(strPropertyName)
        Return wPropertyDescriptor
    End Function

    Private Sub DGV_CellFormatting(sender As Object, e As DataGridViewCellFormattingEventArgs) Handles DGV.CellFormatting
        Dim wRowObj = DGV.Rows(e.RowIndex).DataBoundItem
        Dim wColProps = mColumnsProperties(e.ColumnIndex)

        If wColProps.Name.Equals("NAZIONE", StringComparison.OrdinalIgnoreCase) Then
            e.Value = GetImmagine(DGV.Rows(e.RowIndex).Cells(e.ColumnIndex).Value.ToString)
        End If

    End Sub

    Private Function GetImmagine(Valore As String) As System.Drawing.Image
        Select Case Valore
            Case "A" : Return mImageArgentina
            Case "B" : Return mImageBrasile
            Case "I" : Return mImageItalia
            Case Else : Return Nothing
        End Select
    End Function

    Public Shared Function GetImage(Nome As String) As System.Drawing.Image
        Return GetImage(System.Reflection.Assembly.GetExecutingAssembly, Nome)
    End Function

    Public Shared Function GetImage(Modulo As System.Reflection.Assembly, Nome As String) As System.Drawing.Image
        Dim wNomeAssembly = IO.Path.GetFileNameWithoutExtension(Modulo.Modules(0).Name)
        Dim wNome = String.Concat(wNomeAssembly, ".", Nome)
        Using wStream = Modulo.GetManifestResourceStream(wNome)
            If wStream Is Nothing Then Return Nothing
            Return System.Drawing.Image.FromStream(wStream)
        End Using
    End Function

End Class

Public Class Anagrafiche
    Inherits System.Collections.Generic.List(Of Anagrafica)

    Public Overloads Sub Add(Cognome As String, Nome As String, Ruolo As Anagrafica.RuoloEnum)
        Me.Add(Cognome, Nome, Ruolo, "I")
    End Sub

    Public Overloads Sub Add(Cognome As String, Nome As String, Ruolo As Anagrafica.RuoloEnum, Nazione As String)
        Dim wAnagrafica As New Anagrafica(Cognome, Nome, Ruolo, Nazione)
        Me.Add(wAnagrafica)
    End Sub

    Public Sub Carica()
        Me.Clear()

        Me.Add("Buffon", "Gianluigi", Anagrafica.RuoloEnum.P)
        Me.Add("Neto", "Norberto", Anagrafica.RuoloEnum.P, "B")
        Me.Add("Chiellini", "Giorgio", Anagrafica.RuoloEnum.D)
        Me.Add("Barzagli", "Andrea", Anagrafica.RuoloEnum.D)
        Me.Add("Alves", "Daniel", Anagrafica.RuoloEnum.D, "B")
        Me.Add("Bonucci", "Leonardo", Anagrafica.RuoloEnum.D)
        Me.Add("Sandro", "Alex", Anagrafica.RuoloEnum.D, "B")

        Me.Add("Marchisio", "Claudio", Anagrafica.RuoloEnum.C)
        Me.Add("Khedira", "Sami", Anagrafica.RuoloEnum.C)

        Me.Add("Higuain", "Gonzalo", Anagrafica.RuoloEnum.A, "A")
        Me.Add("Dybala", "Paulo", Anagrafica.RuoloEnum.A, "A")

        Me.Add("Totti", "Francesco", Anagrafica.RuoloEnum.A)
        Me.Add("De Rossi", "Daniele", Anagrafica.RuoloEnum.C)
        Me.Add("Florenzi", "Alessandro", Anagrafica.RuoloEnum.D)


    End Sub

End Class

Public Class Anagrafica
    Public Property Cognome As String
    Public Property Nome As String

    Public Property Ruolo As RuoloEnum

    Public Property Nazione As String

    Public Sub New(Cognome As String, Nome As String, Ruolo As RuoloEnum)
        Me.New(Cognome, Nome, Ruolo, "I")
    End Sub

    Public Sub New(Cognome As String, Nome As String, Ruolo As RuoloEnum, Nazione As String)
        Me.Cognome = Cognome
        Me.Nome = Nome
        Me.Ruolo = Ruolo
        Me.Nazione = Nazione
    End Sub

    Public Enum RuoloEnum
        <Description("Portiere")> P = 1
        <Description("Difensore")> D = 2
        <Description("Centrocampista")> C = 3
        <Description("Portiere")> A = 4
    End Enum

End Class